@@ -140,7 +140,6 @@ secret_collection_init (SecretCollection *self)
 
 	g_mutex_init (&self->pv->mutex);
 	self->pv->cancellable = g_cancellable_new ();
-	self->pv->items = items_table_new ();
 	self->pv->constructing = TRUE;
 }
 
@@ -260,29 +259,13 @@ secret_collection_finalize (GObject *obj)
 		                              (gpointer *)&self->pv->service);
 
 	g_mutex_clear (&self->pv->mutex);
-	g_hash_table_destroy (self->pv->items);
+	if (self->pv->items)
+		g_hash_table_destroy (self->pv->items);
 	g_object_unref (self->pv->cancellable);
 
 	G_OBJECT_CLASS (secret_collection_parent_class)->finalize (obj);
 }
 
-static SecretItem *
-collection_lookup_item (SecretCollection *self,
-                        const gchar *path)
-{
-	SecretItem *item = NULL;
-
-	g_mutex_lock (&self->pv->mutex);
-
-	item = g_hash_table_lookup (self->pv->items, path);
-	if (item != NULL)
-		g_object_ref (item);
-
-	g_mutex_unlock (&self->pv->mutex);
-
-	return item;
-}
-
 static void
 collection_update_items (SecretCollection *self,
                          GHashTable *items)
@@ -296,7 +279,9 @@ collection_update_items (SecretCollection *self,
 	self->pv->items = items;
 	g_mutex_unlock (&self->pv->mutex);
 
-	g_hash_table_unref (previous);
+	if (previous != NULL)
+		g_hash_table_unref (previous);
+
 	g_object_notify (G_OBJECT (self), "items");
 }
 
@@ -524,6 +509,24 @@ secret_collection_class_init (SecretCollectionClass *klass)
 	g_type_class_add_private (gobject_class, sizeof (SecretCollectionPrivate));
 }
 
+static gboolean
+collection_ensure_for_flags_sync (SecretCollection *self,
+                                  SecretCollectionFlags flags,
+                                  GCancellable *cancellable,
+                                  GError **error)
+{
+	SecretCollectionFlags want_flags;
+
+	want_flags = flags & ~secret_collection_get_flags (self);
+
+	if (want_flags & SECRET_COLLECTION_LOAD_ITEMS) {
+		if (!secret_collection_load_items_sync (self, cancellable, error))
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 secret_collection_initable_init (GInitable *initable,
                                  GCancellable *cancellable,
@@ -555,10 +558,8 @@ secret_collection_initable_init (GInitable *initable,
 			collection_take_service (self, service);
 	}
 
-	if (self->pv->init_flags & SECRET_COLLECTION_LOAD_ITEMS) {
-		if (!secret_collection_load_items_sync (self, cancellable, error))
-			return FALSE;
-	}
+	if (!collection_ensure_for_flags_sync (self, self->pv->init_flags, cancellable, error))
+		return FALSE;
 
 	self->pv->constructing = FALSE;
 	return TRUE;
@@ -585,9 +586,9 @@ init_closure_free (gpointer data)
 }
 
 static void
-on_init_items (GObject *source,
-               GAsyncResult *result,
-               gpointer user_data)
+on_ensure_items (GObject *source,
+                 GAsyncResult *result,
+                 gpointer user_data)
 {
 	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
 	SecretCollection *self = SECRET_COLLECTION (source);
@@ -603,16 +604,20 @@ on_init_items (GObject *source,
 static void
 collection_ensure_for_flags_async (SecretCollection *self,
                                    SecretCollectionFlags flags,
+                                   GCancellable *cancellable,
                                    GSimpleAsyncResult *async)
 {
-	InitClosure *init = g_simple_async_result_get_op_res_gpointer (async);
+	SecretCollectionFlags want_flags;
 
-	if (flags & SECRET_COLLECTION_LOAD_ITEMS)
-		secret_collection_load_items (self, init->cancellable,
-		                              on_init_items, g_object_ref (async));
+	want_flags = flags & ~secret_collection_get_flags (self);
 
-	else
+	if (want_flags & SECRET_COLLECTION_LOAD_ITEMS) {
+		secret_collection_load_items (self, cancellable,
+		                              on_ensure_items, g_object_ref (async));
+
+	} else {
 		g_simple_async_result_complete (async);
+	}
 }
 
 static void
@@ -622,13 +627,15 @@ on_init_service (GObject *source,
 {
 	GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
 	SecretCollection *self = SECRET_COLLECTION (g_async_result_get_source_object (user_data));
+	InitClosure *init = g_simple_async_result_get_op_res_gpointer (async);
 	SecretService *service;
 	GError *error = NULL;
 
 	service = secret_service_get_finish (result, &error);
 	if (error == NULL) {
 		collection_take_service (self, service);
-		collection_ensure_for_flags_async (self, self->pv->init_flags, async);
+		collection_ensure_for_flags_async (self, self->pv->init_flags,
+		                                   init->cancellable, async);
 
 	} else {
 		g_simple_async_result_take_error (async, error);
@@ -666,7 +673,8 @@ on_init_base (GObject *source,
 		                    on_init_service, g_object_ref (res));
 
 	} else {
-		collection_ensure_for_flags_async (self, self->pv->init_flags, res);
+		collection_ensure_for_flags_async (self, self->pv->init_flags,
+		                                   init->cancellable, res);
 	}
 
 	g_object_unref (res);
@@ -814,7 +822,7 @@ secret_collection_load_items (SecretCollection *self,
 
 	g_variant_iter_init (&iter, paths);
 	while (g_variant_iter_loop (&iter, "&o", &path)) {
-		item = collection_lookup_item (self, path);
+		item = _secret_collection_find_item_instance (self, path);
 
 		/* No such collection yet create a new one */
 		if (item == NULL) {
@@ -904,7 +912,7 @@ secret_collection_load_items_sync (SecretCollection *self,
 
 	g_variant_iter_init (&iter, paths);
 	while (g_variant_iter_next (&iter, "&o", &path)) {
-		item = collection_lookup_item (self, path);
+		item = _secret_collection_find_item_instance (self, path);
 
 		/* No such collection yet create a new one */
 		if (item == NULL) {
@@ -1777,12 +1785,13 @@ secret_collection_get_flags (SecretCollection *self)
 GList *
 secret_collection_get_items (SecretCollection *self)
 {
-	GList *l, *items;
+	GList *l, *items = NULL;
 
 	g_return_val_if_fail (SECRET_IS_COLLECTION (self), NULL);
 
 	g_mutex_lock (&self->pv->mutex);
-	items = g_hash_table_get_values (self->pv->items);
+	if (self->pv->items)
+		items = g_hash_table_get_values (self->pv->items);
 	for (l = items; l != NULL; l = g_list_next (l))
 		g_object_ref (l->data);
 	g_mutex_unlock (&self->pv->mutex);
@@ -1794,10 +1803,11 @@ SecretItem *
 _secret_collection_find_item_instance (SecretCollection *self,
                                        const gchar *item_path)
 {
-	SecretItem *item;
+	SecretItem *item = NULL;
 
 	g_mutex_lock (&self->pv->mutex);
-	item = g_hash_table_lookup (self->pv->items, item_path);
+	if (self->pv->items)
+		item = g_hash_table_lookup (self->pv->items, item_path);
 	if (item != NULL)
 		g_object_ref (item);
 	g_mutex_unlock (&self->pv->mutex);
@@ -1992,6 +2002,7 @@ secret_collection_get_modified (SecretCollection *self)
 typedef struct {
 	GCancellable *cancellable;
 	gchar *alias;
+	SecretCollectionFlags flags;
 	SecretCollection *collection;
 } ReadClosure;
 
@@ -2046,12 +2057,15 @@ on_read_alias_path (GObject *source,
 			read->collection = _secret_service_find_collection_instance (self,
 			                                                             collection_path);
 			if (read->collection != NULL) {
-				g_simple_async_result_complete (async);
+
+				/* Make sure collection has necessary flags */
+				collection_ensure_for_flags_async (read->collection, read->flags,
+				                                   read->cancellable, async);
 
 			/* No collection loaded, but valid path, load */
 			} else {
 				secret_collection_new_for_dbus_path (self, collection_path,
-				                                     SECRET_COLLECTION_NONE,
+				                                     read->flags,
 				                                     read->cancellable,
 				                                     on_read_alias_collection,
 				                                     g_object_ref (async));
@@ -2095,6 +2109,7 @@ on_read_alias_service (GObject *source,
  * secret_collection_for_alias:
  * @service: (allow-none): a secret service object
  * @alias: the alias to lookup
+ * @flags: options for the collection initialization
  * @cancellable: (allow-none): optional cancellation object
  * @callback: called when the operation completes
  * @user_data: data to pass to the callback
@@ -2110,6 +2125,7 @@ on_read_alias_service (GObject *source,
 void
 secret_collection_for_alias (SecretService *service,
                              const gchar *alias,
+                             SecretCollectionFlags flags,
                              GCancellable *cancellable,
                              GAsyncReadyCallback callback,
                              gpointer user_data)
@@ -2126,6 +2142,7 @@ secret_collection_for_alias (SecretService *service,
 	read = g_slice_new0 (ReadClosure);
 	read->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
 	read->alias = g_strdup (alias);
+	read->flags = flags;
 	g_simple_async_result_set_op_res_gpointer (async, read, read_closure_free);
 
 	if (service == NULL) {
@@ -2173,6 +2190,7 @@ secret_collection_for_alias_finish (GAsyncResult *result,
  * secret_collection_for_alias_sync:
  * @service: (allow-none): a secret service object
  * @alias: the alias to lookup
+ * @flags: options for the collection initialization
  * @cancellable: (allow-none): optional cancellation object
  * @error: location to place error on failure
  *
@@ -2189,6 +2207,7 @@ secret_collection_for_alias_finish (GAsyncResult *result,
 SecretCollection *
 secret_collection_for_alias_sync (SecretService *service,
                                   const gchar *alias,
+                                  SecretCollectionFlags flags,
                                   GCancellable *cancellable,
                                   GError **error)
 {
@@ -2213,11 +2232,19 @@ secret_collection_for_alias_sync (SecretService *service,
 		collection = _secret_service_find_collection_instance (service,
 		                                                       collection_path);
 
+		if (collection != NULL) {
+
+			/* Have a collection with all necessary flags */
+			if (!collection_ensure_for_flags_sync (collection, flags,
+			                                       cancellable, error)) {
+				g_object_unref (collection);
+				collection = NULL;
+			}
+
 		/* No collection loaded, but valid path, load */
-		if (collection == NULL) {
+		} else {
 			collection = secret_collection_new_for_dbus_path_sync (service, collection_path,
-			                                                       SECRET_COLLECTION_LOAD_ITEMS,
-			                                                       cancellable, error);
+			                                                       flags, cancellable, error);
 		}
 	}
 
